PerfectScreenshot = LibStub("AceAddon-3.0"):NewAddon(	"PerfectScreenshot",
														"AceConsole-3.0",
														"AceEvent-3.0");

-- Binding Variables
BINDING_HEADER_PERFECTSCREENSHOT_HEADER = "Perfect Screenshot";
BINDING_NAME_PERFECTSCREENSHOT_SHOOT = "Take Screenshot";

local Options = {
	type = "group",
	get = function(item) return PerfectScreenshot.db.profile[item[#item]] end,
	set = function(item, value) PerfectScreenshot.db.profile[item[#item]] = value end,
	args = {
		viewport = {
			type = "toggle",
			name = "Remove viewport",
			desc = "Resets your viewport to show the whole image.",
			order = 1,
		},
		ui = {
			type = "toggle",
			name = "Hide UI",
			desc = "Hide the User Interface",
			order = 1,
		},
		UnitNameOwn = {
			type = "toggle",
			name = "My Name",
			desc = "Display your character's name in the game world.",
			order = 2,
		},
		UnitNameNPC = {
			type = "toggle",
			name = "NPC Names",
			desc = "Show NPC names in the game world.",
			order = 2
		},
		UnitNameCompanionName = {
			type = "toggle",
			name = "Companions",
			desc = "Show companions' names in the game world.",
			order = 2,
		},
		UnitNamePlayerGuild = {
			type = "toggle",
			name = "Guild Names",
			desc = "Show cplayers' guild names in the game world.",
			order = 2,
		},
		UnitNamePlayerPVPTitle = {
			type = "toggle",
			name = "Titles",
			desc = "Show player titles in the game world.",
			order = 2,
		},
		UnitNameFriendlyPlayerName = {
			type = "toggle",
			name = "Friendly Players",
			desc = "Show friendly players' name in the game world",
			order = 3,
		},
		UnitNameFriendlyPetName = {
			type = "toggle",
			name = "Friendly Players' Pets & Minions",
			desc = "Show friendly pets and minions' in the game world",
			order = 3,
		},
		UnitNameFriendlyCreationName = {
			type = "toggle",
			name = "Friendly Players' Creations",
			desc = "Show friendly creations' name in the game world",
			order = 3,
		},
		UnitNameEnemyPlayerName = {
			type = "toggle",
			name = "Enemy Players",
			desc = "Show enemy players' name in the game world",
			order = 4,
		},
		UnitNameEnemyPetName = {
			type = "toggle",
			name = "Enemy Players' Pets & Minions",
			desc = "Show enemy pets and minions' in the game world",
			order = 4,
		},
		UnitNameEnemyCreationName = {
			type = "toggle",
			name = "Enemy Players' Creations",
			desc = "Show enemy creations' name in the game world",
			order = 4,
		},
	},
}

local Defaults = {
	profile = {
		viewport = true,
		ui = true,
		UnitNameOwn = true,
		UnitNameNPC = true,
		UnitNameCompanionName = false,
		UnitNamePlayerGuild = true,
		UnitNamePlayerPVPTitle = false,
		UnitNameFriendlyPlayerName = true,
		UnitNameFriendlyPetName = false,
		UnitNameFriendlyCreationName = false,
		UnitNameEnemyPlayerName = true,
		UnitNameEnemyPetName = false,
		UnitNameEnemyCreationName = false,
	},
};

local SlashOptions = {
	type = "group",
	handler = PerfectScreenshot,
	args = {
		config = {
			type = "execute",
			desc = "Open configuration dialog",
			name = "Configure",
			func = function()
				InterfaceOptionsFrame_OpenToCategory(PerfectScreenshot.optionFrames.profiles);
				InterfaceOptionsFrame_OpenToCategory(PerfectScreenshot.optionFrames.main);
			end,
		},
		take = {
			type = "execute",
			desc = "Take Screenshot",
			name = "Screenshot",
			func = "TakeScreenshot",
		},
	},
};

local SlashCmds = {
  "ps",
  "perfectscreenshot",
};

local PerfectScreenshot = PerfectScreenshot;

function PerfectScreenshot:TakeScreenshot(_)
	if self.db.profile.viewport then
		-- Remove Viewport
		point, relativeTo, relativePoint, self.TLxOfs, self.TLyOfs = WorldFrame:GetPoint(1);
		point, relativeTo, relativePoint, self.TRxOfs, self.TRyOfs = WorldFrame:GetPoint(2);
		point, relativeTo, relativePoint, self.BLxOfs, self.BLyOfs = WorldFrame:GetPoint(3);
		point, relativeTo, relativePoint, self.BRxOfs, self.BRyOfs = WorldFrame:GetPoint(4);

		WorldFrame:SetPoint("TOPLEFT", 0, 0);
		WorldFrame:SetPoint("TOPRIGHT", 0, 0);
		WorldFrame:SetPoint("BOTTOMLEFT", 0, 0);
		WorldFrame:SetPoint("BOTTOMRIGHT", 0, 0);
			
		-- Special case to handel SunnArt Viewport Frames
		if IsAddOnLoaded("SunnArt") then
			SunnArt:Disable();
		end
	end

	-- Get Unitnames
	self.UnitNameCompanionName = GetCVar("UnitNameCompanionName");
	self.UnitNameEnemyCreationName = GetCVar("UnitNameEnemyCreationName");
	self.UnitNameEnemyPetName = GetCVar("UnitNameEnemyPetName");
	self.UnitNameEnemyPlayerName = GetCVar("UnitNameEnemyPlayerName");
	self.UnitNameFriendlyCreationName = GetCVar("UnitNameFriendlyCreationName");
	self.UnitNameFriendlyPetName = GetCVar("UnitNameFriendlyPetName");
	self.UnitNameFriendlyPlayerName = GetCVar("UnitNameFriendlyPlayerName");
	self.UnitNameNPC = GetCVar("UnitNameNPC");
	self.UnitNameOwn = GetCVar("UnitNameOwn");
	self.UnitNamePlayerGuild = GetCVar("UnitNamePlayerGuild");
	self.UnitNamePlayerPVPTitle = GetCVar("UnitNamePlayerPVPTitle");
	
	SetCVar("UnitNameCompanionName", self.db.profile.UnitNameCompanionName and 1 or 0);
	SetCVar("UnitNameEnemyCreationName",  self.db.profile.UnitNameEnemyCreationName and 1 or 0);
	SetCVar("UnitNameEnemyPetName", self.db.profile.UnitNameEnemyPetName and 1 or 0);
	SetCVar("UnitNameEnemyPlayerName", self.db.profile.UnitNameEnemyPlayerName and 1 or 0);
	SetCVar("UnitNameFriendlyCreationName", self.db.profile.UnitNameFriendlyCreationName and 1 or 0);
	SetCVar("UnitNameFriendlyPetName", self.db.profile.UnitNameFriendlyPetName and 1 or 0);
	SetCVar("UnitNameFriendlyPlayerName", self.db.profile.UnitNameFriendlyPlayerName and 1 or 0);
	SetCVar("UnitNameNPC", self.db.profile.UnitNameNPC and 1 or 0);
	SetCVar("UnitNameOwn", self.db.profile.UnitNameOwn and 1 or 0);
	SetCVar("UnitNamePlayerGuild", self.db.profile.UnitNamePlayerGuild and 1 or 0);
	SetCVar("UnitNamePlayerPVPTitle", self.db.profile.UnitNamePlayerPVPTitle and 1 or 0);
	--PerfectScreenshot:Print(self.UnitNameNPC);
	-- Hide Frames
	if self.db.profile.ui then
		UIParent:Hide();
	end
	
	self.screenshot = 1;
	
	--Take Screenshot
	Screenshot();
end

function PerfectScreenshot:RestoreUI()
	if self.screenshot then
		if self.db.profile.viewport then
			-- Restore Viewport
			WorldFrame:SetPoint("TOPLEFT", self.TLxOfs, self.TLyOfs);
			WorldFrame:SetPoint("TOPRIGHT", self.TRxOfs, self.TRyOfs);
			WorldFrame:SetPoint("BOTTOMLEFT", self.BLxOfs, self.BLyOfs);
			WorldFrame:SetPoint("BOTTOMRIGHT", self.BRxOfs, self.BRyOfs);
			
			-- Special case to handel SunnArt Viewport Frames
			if IsAddOnLoaded("SunnArt") then
				SunnArt:Enable();
			end
		end
		
		-- Restore Unitnames
		SetCVar("UnitNameCompanionName", self.UnitNameCompanionNam);
		SetCVar("UnitNameEnemyCreationName", self.UnitNameEnemyCreationName);
		SetCVar("UnitNameEnemyPetName", self.UnitNameEnemyPetName);
		SetCVar("UnitNameEnemyPlayerName", self.UnitNameEnemyPlayerName);
		SetCVar("UnitNameFriendlyCreationName", self.UnitNameFriendlyCreationName);
		SetCVar("UnitNameFriendlyPetName", self.UnitNameFriendlyPetName);
		SetCVar("UnitNameFriendlyPlayerName", self.UnitNameFriendlyPlayerName);
		SetCVar("UnitNameNPC", self.UnitNameNPC);
		SetCVar("UnitNameOwn", self.UnitNameOwn);
		SetCVar("UnitNamePlayerGuild", self.UnitNamePlayerGuild);
		SetCVar("UnitNamePlayerPVPTitle", self.UnitNamePlayerPVPTitle);

		-- Show Frames
		if self.db.profile.ui then
			UIParent:Show();
		end
		
		self.screenshot = nil;
		
		--PerfectScreenshot:Print("Screenshot taken");
	end
end

function PerfectScreenshot:SCREENSHOT_FAILED()
	PerfectScreenshot:RestoreUI();
end

function PerfectScreenshot:SCREENSHOT_SUCCEEDED()
	PerfectScreenshot:RestoreUI();
end

function PerfectScreenshot:OnInitialize()
	-- Load our database.
	self.db = LibStub("AceDB-3.0"):New("PerfectScreenshotDB", Defaults, "Default");

	-- Set up our config options.
	local profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db);
  
	local config = LibStub("AceConfig-3.0");
	config:RegisterOptionsTable("PerfectScreenshot", SlashOptions, SlashCmds);

	local registry = LibStub("AceConfigRegistry-3.0");
	registry:RegisterOptionsTable("PerfectScreenshot Options", Options);
	registry:RegisterOptionsTable("PerfectScreenshot Profiles", profiles);

	local dialog = LibStub("AceConfigDialog-3.0");
	self.optionFrames = {
		main = dialog:AddToBlizOptions(	"PerfectScreenshot Options", "PerfectScreenshot"),
		profiles = dialog:AddToBlizOptions(	"PerfectScreenshot Profiles", "Profiles",
											"PerfectScreenshot");
	};

	self:RegisterEvent("SCREENSHOT_FAILED");
	self:RegisterEvent("SCREENSHOT_SUCCEEDED");

	--PerfectScreenshot:Print("PerfectScreenshot loaded");
end
